#pragma once

//#define _USE_MATH_DEFINES

#include <stdint.h>
#include <exception>
#include <csrmem.h>
//#include <cmath>

#ifndef NULL
#define NULL 0
#endif

#ifndef _INC_STRING
#include <string.h>
#include <csrstr.h>
#endif

#ifdef min
#undef min
#undef max
#endif

/*
#define MATH_ALLOC(T)			(T*) csrmath_thread_context.alloc( sizeof(T) )
#define MATH_ALLOCS(T,count)	(T*) csrmath_thread_context.alloc( (count)*sizeof(T) )
#define MATH_FREE(ref)			csrmath_thread_context.free(ref)
*/
#define MATH_ALLOC(T)			(T*) csrutils::heap_alloc<T>()
#define MATH_ALLOCS(T,count)	(T*) csrutils::heap_alloc<T>(count)
#define MATH_FREE(ref)			csrutils::heap_free(ref)


namespace csrmath {

typedef unsigned __int64 uintx_t;
typedef __int64 intx_t;

template<typename T> T min(T x, T y) {
	if( x < y ) return x;
	else return y;
}

template<typename T> T max(T x, T y) {
	if( x > y ) return x;
	else return y;
}

template<typename T, size_t count> T min( const T (&x)[count] ) {
	T min_val = x[0];
	for( size_t i=1; i<count; i++ ) {
		if( min_val > x[i] )
			min_val = x[i];
	}
	return min_val;
}

template<typename T, size_t count> T max( const T (&x)[count] ) {
	T max_val = x[0];
	for( size_t i=1; i<count; i++ ) {
		if( max_val < x[i] )
			max_val = x[i];
	}
	return max_val;
}

template<typename T> T abs(T x) {
	if( x >= 0 ) return x;
	else return -x;
}

inline double abs(double x) {
	return x > 0.0 ? x : - x;
}

double sqrt(double);
double cbrt(double);
double exp(double);
double log(double);
double log10(double);
double log2(double);
double pow(double x, double y);

double sin(double);
double cos(double);
double tan(double);

double asin(double);
double acos(double);
double atan(double);
double atan2(double y, double x);

namespace Const {
static const double E         = 2.71828182845904523536;
static const double Log2E     = 1.44269504088896340736;
static const double Log10E    = 0.434294481903251827651;
static const double Ln2       = 0.693147180559945309417;
static const double Ln10      = 2.30258509299404568402;
static const double Pi        = 3.14159265358979323846;
static const double Pi_2      = 1.57079632679489661923;
static const double Pi_4      = 0.785398163397448309616;
static const double u1_PI     = 0.318309886183790671538;
static const double u2_PI     = 0.636619772367581343076;
static const double u2_SqrtPi = 1.12837916709551257390;
static const double Sqrt2     = 1.41421356237309504880;
static const double Sqrt1_2   = 0.707106781186547524401;
}

};

#include <csrmath_engine.h>
