#pragma once

#include <csrmath.h>
#include <csrmath_numeric.h>

namespace csrmath {

struct MathContextInt;
struct MathContext {
	void* (*alloc) (size_t);
	void (*free) (void*);
	MathContextInt *_internal;
};

extern __declspec(thread) MathContext csrmath_thread_context;

class MathEngine;

enum MatrixInverseMethod {
	MATRIX_INVERSE_ADJUGATE,
	MATRIX_INVERSE_LUP,
	MATRIX_INVERSE_GAUSS
};

struct MathEngineImpl_Ctx;
class MathEngineImpl {
	MathEngineImpl_Ctx *ctx;

	MathEngineImpl();
	~MathEngineImpl();

	bool setMatrixInverseMethod(MatrixInverseMethod);

	friend class MathEngine;
};

class MathEngine {
	MathEngineImpl *impl;

public:
	MathEngine() : impl(NULL) { }

	inline bool initEngine() {
		impl = new MathEngineImpl();
		return true;
	}

	inline bool release() {
		delete impl;
		return true;
	}

	inline bool setMatrixInverseMethod(MatrixInverseMethod method) {
		return impl->setMatrixInverseMethod(method);
	}
};

};
