#pragma once

#include <csrmath_numeric.h>

namespace csrmath {

class Rational {
public:
	intx_t nom;
	intx_t den;

	inline Rational() : nom(0), den(1) { }
	inline Rational(intx_t n) : nom(n), den(1) { }
	inline Rational(intx_t n, intx_t d) : nom(n), den(d) {
		intx_t gcd = GCD(n,d);
		nom /= gcd;
		den /= gcd;
		if( den < 0 ) {
			nom = - nom;
			den = - den;
		}
	}
	inline Rational(const Rational& r) : nom(r.nom), den(r.den) { }
	//inline Rational(const volatile Rational& r) : nom(r.nom), den(r.den) { }

	inline operator double() const { return (double) nom / den; }
	inline operator float() const { return (float) nom / den; }

	inline Rational operator+ (const Rational& r1) const {
		return Rational( nom*r1.den + r1.nom*den, den*r1.den );
	}

	inline Rational operator- (const Rational& r1) const {
		return Rational( nom*r1.den - r1.nom*den, den*r1.den );
	}

	inline Rational operator* (const Rational& r1) const {
		return Rational( nom*r1.nom, den*r1.den );
	}

	inline Rational operator/ (const Rational& r1) const {
		return Rational( nom*r1.den, den*r1.nom );
	}

	inline Rational operator+ (int rarg) const {
		return Rational( nom + rarg*den, den );
	}

	inline Rational operator- (int rarg) const {
		return Rational( nom - rarg*den, den );
	}

	inline Rational operator* (int rarg) const {
		return Rational( nom*rarg, den );
	}

	inline Rational operator/ (int rarg) const {
		return Rational( nom, den*rarg );
	}

	inline Rational operator- () const {
		return Rational(-nom,den);
	}

	inline bool operator== (const Rational& rarg) const {
		return nom==rarg.nom && den==rarg.den;
	}

	inline bool operator> (const Rational& rarg) const {
		intx_t lnom = nom * rarg.den;
		intx_t rnom = rarg.nom * den;
		return lnom > rnom;
	}

	inline bool operator< (const Rational& rarg) const {
		intx_t lnom = nom * rarg.den;
		intx_t rnom = rarg.nom * den;
		return lnom < rnom;
	}

	inline bool operator<= (const Rational& rarg) const {
		return !( *this > rarg );
	}

	inline bool operator>= (const Rational& rarg) const {
		return !( *this < rarg );
	}
};

inline Rational operator+ (int larg, const Rational& rarg) {
	return rarg + larg;
}

inline Rational operator- (int larg, const Rational& rarg) {
	return (-rarg) + larg;
}

inline Rational operator* (int larg, const Rational& rarg) {
	return larg * rarg;
}

inline Rational operator/ (int larg, const Rational& rarg) {
	return Rational(larg) / rarg;
}

};
