#pragma once

#include <csrmath.h>

namespace csrmath {

class VectorD {
	size_t t_size;
	double *t_data;

public:
	inline VectorD() : t_size(0), t_data(NULL) { }
	inline VectorD(size_t size)
			: t_size(size), t_data( MATH_ALLOCS(double,size) ) {
		for( size_t i=0; i<size; i++ )
			t_data[i] = 0.0;
	}
	inline VectorD(size_t size, const double *data)
			: t_size(size), t_data( MATH_ALLOCS(double,size) ) {
		memcpy(t_data,data,size*sizeof(double));
	}
	template<size_t size>
	VectorD(double (&data)[size])
			: t_size(size), t_data( MATH_ALLOCS(double,size) ) {
		memcpy(t_data,data,size*sizeof(double));
	}
	inline VectorD(const VectorD& v)
		: t_size(v.t_size), t_data( MATH_ALLOCS(double,v.t_size) ) {
		memcpy(t_data,v.t_data,t_size*sizeof(double));
	}
	inline ~VectorD() {
		if( t_size > 0 )
			MATH_FREE(t_data);
	}

	inline VectorD& operator= (const VectorD& v) {
		if( t_size == v.t_size ) {
			memcpy(t_data,v.t_data,t_size*sizeof(double));
			return *this;
		}
		if( t_size > 0 )
			MATH_FREE(t_data);
		t_size = v.t_size;
		t_data = MATH_ALLOCS(double,v.t_size);
		memcpy(t_data,v.t_data,t_size*sizeof(double));
		return *this;
	}

	inline size_t size() const { return t_size; }
	inline double& operator[] (size_t n) { return t_data[n]; }
	inline double operator[] (size_t n) const { return t_data[n]; }
};

};
