#include <Windows.h>
#include <stdio.h>
#include <csrmem.h>
#include <csrheap.h>

#include "csrmath.h"
#include "csrmath_int.h"

#define THCTX csrmath::csrmath_thread_context

typedef void (*mem_free) (void*);

using namespace csrmath;

__declspec(thread) csrmath::MathContext csrmath::csrmath_thread_context;

bool InitMathThreadContext(MathEngineImpl* impl, MathEngineImpl_Ctx *impl_ctx);

MathEngineImpl::MathEngineImpl() {
	ctx = new( heap_alloc<MathEngineImpl_Ctx>() ) MathEngineImpl_Ctx;
	InitMathThreadContext(this,ctx);
	heap_set(ctx->heap);
}

MathEngineImpl::~MathEngineImpl() {
	delete ctx;
}

bool MathEngineImpl::setMatrixInverseMethod(MatrixInverseMethod method) {
	THCTX._internal->matrixInverseMethod = method;
	return true;
}

void math_free_local(void* ref) {
	HeapFree( THCTX._internal->heap, 0, ref );
}

void* math_alloc(size_t sz) {
	//return malloc(sz);
	size_t *ref = (size_t*) HeapAlloc( THCTX._internal->heap, HEAP_ZERO_MEMORY, sz+sizeof(size_t) );
	//printf("alloc at %p\n",ref);
	*ref = (size_t) math_free_local;
	return ref+1;
}

void math_free(void *ref) {
	size_t *rrf = (size_t*) ref;
	//return free(ref);
	//printf("free  at %p\n",ref);
	mem_free func_free = * (mem_free*) (rrf-1);
	func_free(rrf-1);
	//HeapFree( THCTX.internal->heap, 0, ref );
}

bool InitMathThreadContext(MathEngineImpl* impl, MathEngineImpl_Ctx *impl_ctx) {
	THCTX.alloc = math_alloc;
	THCTX.free  = math_free;
	THCTX._internal = new MathContextInt;
	THCTX._internal->heap = HeapCreate(0,0x10000,0x1000000);
	THCTX._internal->matrixInverseMethod = MATRIX_INVERSE_LUP;
	return true;
}