#include "csrmath_matrix.h"
#include "csrmath_int.h"

using namespace csrmath;

struct csrmath::MatrixD_Impl {
	MatrixD *lu;
};

void csrmath::MatrixD_Impl_free(MatrixD_Impl* impl) {
	delete impl;
}

void matrix_lup_decomposition(const MatrixD& mat, MatrixD& L, MatrixD& U, MatrixD& P) {

}

void matrix_lu_decomposition(const MatrixD& A, MatrixD& L, MatrixD& U) {
	size_t N = A.getRows();
	size_t i, j, k;
	for( j=0; j<N; j++ )
		U.get(0,j) = A.get(0,j);
	for( j=1; j<N; j++ )
		L.get(j,0) = A.get(j,0) / U.get(0,0);
	for( i=1; i<N; i++ ) {
		for( j=i; j<N; j++ ) {
			double S = A.get(i,j);
			for( k=0; k<i; k++ )
				S -= L.get(i,k) * U.get(k,j);
			U.get(i,j) = S;
		}
		for( j=i+1; j<N; j++ ) {
			double S = A.get(j,i);
			for( k=0; k<i; k++ )
				S -= L.get(j,k) * U.get(k,i);
			L.get(j,i) = S / U.get(i,i);
		}
	}
	for( i=0; i<N; i++ )
		L.get(i,i) = 1.0;
}

void matrix_inverse_lup(MatrixD& res, const MatrixD& mat) {
	size_t H = mat.getRows();
	MatrixD L(H,H), U(H,H);
	matrix_lu_decomposition(mat,L,U);

}

void matrix_inverse_gauss(MatrixD& res, const MatrixD& mat) {
	// TODO:
	matrix_inverse_lup(res,mat);
}

void matrix_inverse_adjugate(MatrixD& res, const MatrixD& mat) {
	// TODO:
	matrix_inverse_lup(res,mat);
}

double matrix_det_lup(const MatrixD& mat) {
	size_t H = mat.getRows();
	MatrixD L(H,H), U(H,H);
	matrix_lu_decomposition(mat,L,U);
	double detL = 1.0;
	double detU = 1.0;
	for( size_t i=0; i<H; i++ ) {
		detL *= L.get(i,i);
		detU *= U.get(i,i);
	}
	return detL * detU;
}

void MatrixD::decomposeLU(MatrixD& L, MatrixD& U) {
	L = MatrixD(H,H);
	U = MatrixD(H,H);
	matrix_lu_decomposition(*this,L,U);
}

MatrixD MatrixD::inverse() const {
	if( H != W )
		throw MatrixNonSquare();
	MatrixD res(H,W);
	switch( csrmath_thread_context._internal->matrixInverseMethod ) {
	case MATRIX_INVERSE_ADJUGATE:
		matrix_inverse_adjugate(res,*this);
		break;
	case MATRIX_INVERSE_LUP:
		matrix_inverse_adjugate(res,*this);
		break;
	case MATRIX_INVERSE_GAUSS:
		matrix_inverse_adjugate(res,*this);
		break;
	}
	return res;
}

double MatrixD::det() const {
	if( H != W )
		throw MatrixNonSquare();
	return matrix_det_lup(*this);
}

void MatrixD::rowReduce() {
}
