#include <iostream>

#include <csrmath.h>
#include <csrmath_complex.h>
#include <csrmath_rational.h>
#include <csrmath_functions.h>
#include <csrmath_matrix.h>
#include <csrmath_tmatrix.h>
#include <csrmath_polynomial.h>
#include <csrmath_vector.h>
#include <csrmath_minimize.h>
#include <csrmath_avx.h>
//#include <array>

using namespace std;
using namespace csrmath;

class SomeFunction : public MinimizableFunction {
	double eval(double x, double y) {
		return x*x - 1.1*x + y*y + 2*y;
	}
	double operator() (const VectorD& args) {
		return eval(args[0],args[1]);
	}
};

void test_matrix() {
	for( int n=1; n<10; n++ ) {
		MatrixD A(n,n);
		for( int i=0; i<n; i++ )
		for( int j=0; j<n; j++ )
			A.get(i,j) = rand() / 100.0;
		MatrixD L, U;
		A.decomposeLU(L,U);
		MatrixD Ax = L * U;
		if( Ax != A ) {
			std::cout << "Faild at n=" << n << std::endl;
			std::cout << "A=\n" << A << "LU=\n" << Ax << "L=\n" << L << "U=\n" << U;
			std::cout << "------------------\n";
		}
	}
}

void test_functions() {
	/*double d_erf[5] = { csrmath::erf(0.0), csrmath::erf(0.25), csrmath::erf(0.5), csrmath::erf(0.75), csrmath::erf(1.0) };
	//double a_erf[5] = { ::erf(0.0), ::erf(0.25), ::erf(0.5), ::erf(0.75), ::erf(1.0) };
	ComplexD z = expi(2.0);
	std::cout << z.Re();*/
	float src[64], dst1[64],dst2[64];
	for( size_t i=0; i<64; i++ ) {
		src[i] = 2*Const::Pi * i / 64.0 -5.0;
		dst1[i] = ::sin(src[i]);
	}
	for( size_t i=0; i<16; i++ )
		*(__m128*)(dst2+4*i) = __m128_sin_ps( *(__m128*)(src+4*i) );
	for( size_t i=0; i<64; i++ )
		std::cout << src[i] << " " << dst1[i] << '=' << dst2[i] << '\n';
}

void main() {
	MathEngine engine;
	engine.initEngine();

	test_functions();
	//test_matrix();

	/*for(;;) {
		int a, b, c, d;
		cin >> a >> b >> c >> d;
		Rational x(a,b);
		Rational y(c,d);
		Rational z = x + y;
		cout << x.nom << '/' << x.den << " + " << y.nom << '/' << y.den
			<< " = " << z.nom << '/' << z.den << endl;
	}*/

	/*double ks[] = { 3.0, 1.0 };
	double ks2[] = { 3.0, 1.0 };
	Polynomial poly(2,ks);
	Polynomial p1(2,ks2);
	poly = poly * p1;
	for( double x=0.0; x<5.0; x+=1.0 )
		std::cout << "P(" << x << ") = " << poly(x) << std::endl;*/

	/*double vecd[] = { 1, 2, 3 };
	VectorD vec(vecd);
	vec[0] = 5.0;

	Minimizer MM(2);
	SomeFunction mfunc;
	MM.setFunction( &mfunc );
	MM.setArgument(0,-5.0,5.0,0.0);
	MM.setArgument(1,-5.0,5.0,0.0);
	VectorD min_arg = MM.minimize();*/


}
